"""
 Copyright 2015 Cristofor Nogueira Gascons
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = 'cris nogueira'

from modelSMT import ModelBV
from ModelSAT import ModelSAT
import sys

# keywords
model = '-m'
encoding = '-e'
timeout = '-t'
help = '-h'
help_large = '--help'

def parse_args(args):
  options = dict()
  # parse model [default: MaxSMT]
  if model in args:
    index = args.index(model)
    options[model] = args[index+1].lower()
  else:
    options[model] = 'sat'
  # sat encoding (only available for SAT model) [default: kCN]
  if encoding in args:
    index = args.index(encoding)
    options[encoding] = args[index+1].lower()    
  else:
    options[encoding] = 'mcn'
  # timeout [default: 3600 sec]
  if timeout in args:
    index = args.index(timeout)
    options[timeout] = int(args[index+1])*1000
  else:
    options[timeout] = 1800000 # in milliseconds
  return options

def show_help(args):
  print 'usage: %s [options] <instance_xml>' % args[0]
  print 'options:'
  print '\t%s:\t\'sat\' for MaxSAT model (default)' % model
  print '\t   \t\'bv\'  for SMT BitVecor based model'
  print '\t%s:\t\'mcn\' for M-Cardinality Network encoding (default, -m=sat)' % encoding
  print '\t   \t\'mto\' for Modulo Totalizer encoding (-m=sat)'
  print '\t%s:\ttimeout (default -t=1800)' % timeout
  print ''

def main(args):
  global timeout
  global encoding
  global model
  if help in args or help_large in args or len(args) < 2:
    show_help(args)
    return
  options = parse_args(args)
  instance_xml = args[-1]
  if options[model] == 'bv':
    timeout = options[timeout]
    ModelBV(instance_xml, timeout)
  # MaxSAT SOLVER
  else:
    if options[encoding] == 'mto':
      mto = True
    else:
      mto = False
    timeout = options[timeout]
    try:
      ModelSAT(instance_xml, mto, timeout)
    except:
      pass


if __name__ == '__main__':
    main(sys.argv)
