"""
 Copyright 2015 Cristofor Nogueira Gascons
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import xml.etree.ElementTree as ET
import xml.dom.minidom
import re

__author__ = 'cris Nogueira'



class XMLWritter:

    def __init__(self, source_xml, model, identificador='PFG_CristoforNogueira_2015'):

        # append a new solution child to the xml_source
        tree = ET.ElementTree()
        tree.parse(source_xml)
        xml_root = tree.getroot()
        xml_solution_groups = xml_root.find('SolutionGroups')
        if xml_solution_groups is None:
            xml_solution_groups = ET.SubElement(xml_root, 'SolutionGroups')
        xml_solution_group = ET.SubElement(xml_solution_groups, 'SolutionGroup')
        xml_solution_group.set('Id', identificador)

        xml_metadata = ET.SubElement(xml_solution_group, 'MetaData')
        xml_node = ET.SubElement(xml_metadata, 'Contributor')
        xml_node.text = 'Cristofor Nogueira'
        xml_node = ET.SubElement(xml_metadata, 'Date')
        xml_node.text = '17052015'
        xml_node = ET.SubElement(xml_metadata, 'Description')
        xml_node.text = 'Solution obtained with a BitVector modelling powered by z3 theorem-prover'

        xml_solution = ET.SubElement(xml_solution_group, 'Solution')
        xml_solution.set('Reference', xml_root.find('Instances').find('Instance').get('Id'))
        xml_events = ET.SubElement(xml_solution, 'Events')

        self.inflate_events_tag(xml_events, model)

        # write element tree to a file
        outfile = 'output/'+ source_xml[(source_xml.find('/')+1):]
        tree.write(outfile)
        print 'writed in:', outfile

        xml_file = xml.dom.minidom.parse(outfile) # or xml.dom.minidom.parseString(xml_string)
        pretty_xml_as_string = xml_file.toprettyxml()
        res = re.sub(r'\n\s*\n', '\n', pretty_xml_as_string)
        ofile = open(outfile, 'w')
        ofile.write(res)
        ofile.close()

    def inflate_events_tag(self, xml_events, model):

        # get time references from solution
        for event_id, duration, time_id in model.get_times_assignments():
            # create a sub-tag for each sub-event
            event_tag = ET.SubElement(xml_events, 'Event')
            event_tag.set('Reference', event_id)
            duration_tag = ET.SubElement(event_tag, 'Duration')
            duration_tag.text = str(duration)
            time_tag = ET.SubElement(event_tag, 'Time')
            time_tag.set('Reference', time_id)
